/*******************************************************************************

  
  Copyright(c) 1999 - 2008 Intel Corporation. All rights reserved.
  
  This program is free software; you can redistribute it and/or modify it 
  under the terms of the GNU General Public License as published by the Free 
  Software Foundation; either version 2 of the License, or (at your option) 
  any later version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
  more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 
  Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  The full GNU General Public License is included in this distribution in the
  file called LICENSE.
  
  Contact Information:
  e1000 mailing list <e1000-devel@lists.sourceforge.net>
  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497

*******************************************************************************/

#include <stdio.h>
#include "ethregs.h"


extern int clr;
extern int offset;

void printit32_array (int i, char *name, volatile void *mem, int array_len)
{
	int j,n=0, alreadyzero=0;
	char regname[16];

	for (j=i, n=0; n < array_len; n++) {
		if (*(u32 *)(mem + j) == 0) {
			switch (alreadyzero) {
			case 0:
				alreadyzero = 1;
				break;
			case 1:
				printf("\t...\n");
				alreadyzero = 2;
				continue;
			case 2:
				continue;
			}
		} else
			alreadyzero = 0;
		sprintf(regname, "%s[%d]", name, n);
		printit32(j, regname, mem);
		j += sizeof(u32);
	}
}

void dump_82575_registers(volatile void *mem, u32 size)
{
	unsigned int i, j, n;
	char name[16];

	for (i = 0; i < size; i += sizeof(u32)) {
		switch (i & 0xffffffff) {
		case E1000_CTRL:
			sprintf(name, "CTRL"); break;
		case E1000_STATUS:
			sprintf(name, "STATUS"); break;
		case E1000_EECD:
			sprintf(name, "EECD"); break;
		case E1000_EERD:
			sprintf(name, "EERD"); break;
		case E1000_CTRL_EXT:
			sprintf(name, "CTRL_EXT"); break;
		case E1000_FLA:
			sprintf(name, "FLA"); break;
		case E1000_MDIC:
			sprintf(name, "MDIC"); break;
		case E1000_SCTL:
			sprintf(name, "SCTL"); break;
		case E1000_FCAL:
			sprintf(name, "FCAL"); break;
		case E1000_FCAH:
			sprintf(name, "FCAH"); break;
		case E1000_FCT:
			sprintf(name, "FCT"); break;
		case E1000_CONNSW:
			sprintf(name, "CONNSW"); break;
		case E1000_VET:
			sprintf(name, "VET"); break;
		case E1000_RCTL:
			sprintf(name, "RCTL"); break;
		case E1000_FCTTV:
			sprintf(name, "FCTTV"); break;
		case E1000_TXCW:
			sprintf(name, "TXCW"); break;
		case E1000_RXCW:
			sprintf(name, "RXCW"); break;
		case E1000_TCTL:
			sprintf(name, "TCTL"); break;
		case E1000_TCTL_EXT:
			sprintf(name, "TCTL_EXT"); break;
		case E1000_TIPG:
			sprintf(name, "TIPG"); break;
		case E1000_LEDCTL:
			sprintf(name, "LEDCTL"); break;
		case E1000_PBA:
			sprintf(name, "PBA"); break;
		case E1000_PBS:
			sprintf(name, "PBS"); break;
		case E1000_EEMNGCTL:
			sprintf(name, "EEMNGCTL"); break;
		case E1000_EEARBC:
			sprintf(name, "EEARBC"); break;
		case E1000_I2CCMD:
			sprintf(name, "I2CCMD"); break;
		case E1000_I2CPARAMS:
			sprintf(name, "I2CPARAMS"); break;
		case E1000_FLOP:
			sprintf(name, "FLOP"); break;
		case E1000_WDSTP:
			sprintf(name, "WDSTP"); break;
		case E1000_SWDSTS:
			sprintf(name, "SWDSTS"); break;
		case E1000_FRTIMER:
			sprintf(name, "FRTIMER"); break;
		case E1000_TCPTIMER:
			sprintf(name, "TCPTIMER"); break;
		case E1000_ICR:
			if (!clr) continue;
			sprintf(name, "ICR"); break;
		case E1000_ICS:
			sprintf(name, "ICS"); break;
		case E1000_IMS:
			sprintf(name, "IMS"); break;
		case E1000_IMC:
			sprintf(name, "IMC"); break;
		case E1000_IAM:
			sprintf(name, "IAM"); break;
		case E1000_EICR:
			if (!clr) continue;
			sprintf(name, "EICR"); break;
		case E1000_EICS:
			sprintf(name, "EICS"); break;
		case E1000_EIMS:
			sprintf(name, "EIMS"); break;
		case E1000_EIMC:
			sprintf(name, "EIMC"); break;
		case E1000_EIAC:
			sprintf(name, "EIAC"); break;
		case E1000_EIAM:
			sprintf(name, "EIAM"); break;
		case E1000_MSIXBM(0):
			sprintf(name, "MSIXBM");
			printit32_array(i, name, mem, 10);
			continue;
		case E1000_EITR(0):
			sprintf(name, "EITR");
			printit32_array(i, name, mem, 10);
			continue;
		case E1000_FCRTL:
			sprintf(name, "FCRTL"); break;
		case E1000_FCRTH:
			sprintf(name, "FCRTH"); break;
		case E1000_PBRTH:
			sprintf(name, "PBRTH"); break;
		case E1000_FCRTV:
			sprintf(name, "FCRTV"); break;
		case E1000_RDPUMB:
			sprintf(name, "RDPUMB"); break;
		case E1000_RDPUAD:
			sprintf(name, "RDPUAD"); break;
		case E1000_RDPUWD:
			sprintf(name, "RDPUWD"); break;
		case E1000_RDPURD:
			sprintf(name, "RDPURD"); break;
		case E1000_RDPUCTL:
			sprintf(name, "RDPUCTL"); break;
		case E1000_RDBAL(0):
			sprintf(name, "RDBAL0"); break;
		case E1000_RDBAH(0):
			sprintf(name, "RDBAH0"); break;
		case E1000_RDLEN(0):
			sprintf(name, "RDLEN0"); break;
		case E1000_SRRCTL(0):
			sprintf(name, "SRRCTL(0)"); break;
		case E1000_RDH(0):
			sprintf(name, "RDH0"); break;
		case E1000_DCA_RXCTRL(0):
			sprintf(name, "DCA_RXCTRL0"); break;
		case E1000_RDT(0):
			sprintf(name, "RDT0"); break;
		case E1000_RXDCTL(0):
			sprintf(name, "RXDCTL0"); break;
		case E1000_RDBAL(1):
			sprintf(name, "RDBAL1"); break;
		case E1000_RDBAH(1):
			sprintf(name, "RDBAH1"); break;
		case E1000_RDLEN(1):
			sprintf(name, "RDLEN1"); break;
		case E1000_SRRCTL(1):
			sprintf(name, "SRRCTL(1)"); break;
		case E1000_RDH(1):
			sprintf(name, "RDH1"); break;
		case E1000_DCA_RXCTRL(1):
			sprintf(name, "DCA_RXCTRL1"); break;
		case E1000_RDT(1):
			sprintf(name, "RDT1"); break;
		case E1000_RXDCTL(1):
			sprintf(name, "RXDCTL1"); break;
		case E1000_RDBAL(2):
			sprintf(name, "RDBAL2"); break;
		case E1000_RDBAH(2):
			sprintf(name, "RDBAH2"); break;
		case E1000_RDLEN(2):
			sprintf(name, "RDLEN2"); break;
		case E1000_SRRCTL(2):
			sprintf(name, "SRRCTL(2)"); break;
		case E1000_RDH(2):
			sprintf(name, "RDH2"); break;
		case E1000_DCA_RXCTRL(2):
			sprintf(name, "DCA_RXCTRL2"); break;
		case E1000_RDT(2):
			sprintf(name, "RDT2"); break;
		case E1000_RXDCTL(2):
			sprintf(name, "RXDCTL2"); break;
		case E1000_RDBAL(3):
			sprintf(name, "RDBAL3"); break;
		case E1000_RDBAH(3):
			sprintf(name, "RDBAH3"); break;
		case E1000_RDLEN(3):
			sprintf(name, "RDLEN3"); break;
		case E1000_SRRCTL(3):
			sprintf(name, "SRRCTL(3)"); break;
		case E1000_RDH(3):
			sprintf(name, "RDH3"); break;
		case E1000_DCA_RXCTRL(3):
			sprintf(name, "DCA_RXCTRL3"); break;
		case E1000_RDT(3):
			sprintf(name, "RDT3"); break;
		case E1000_RXDCTL(3):
			sprintf(name, "RXDCTL3"); break;
		case E1000_TDFH:
			sprintf(name, "TDFH"); break;
		case E1000_TDFT:
			sprintf(name, "TDFT"); break;
		case E1000_TDFHS:
			sprintf(name, "TDFHS"); break;
		case E1000_TDFPC:
			sprintf(name, "TDFPC"); break;
		case E1000_DTXCTL:
			sprintf(name, "DTXCTL"); break;
		case E1000_TDPUMB:
			sprintf(name, "TDPUMB"); break;
		case E1000_TDPUAD:
			sprintf(name, "TDPUAD"); break;
		case E1000_TDPUWD:
			sprintf(name, "TDPUWD"); break;
		case E1000_TDPURD:
			sprintf(name, "TDPURD"); break;
		case E1000_TDPUCTL:
			sprintf(name, "TDPUCTL"); break;
		case E1000_TDBAL(0):
			sprintf(name, "TDBAL0"); break;
		case E1000_TDBAH(0):
			sprintf(name, "TDBAH0"); break;
		case E1000_TDLEN(0):
			sprintf(name, "TDLEN0"); break;
		case E1000_TDH(0):
			sprintf(name, "TDH0"); break;
		case E1000_DCA_TXCTRL(0):
			sprintf(name, "DCA_TXCTRL0"); break;
		case E1000_TDT(0):
			sprintf(name, "TDT0"); break;
		case E1000_TXDCTL(0):
			sprintf(name, "TXDCTL"); break;
		case E1000_TDWBAL(0):
			sprintf(name, "TDWBAL0"); break;
		case E1000_TDWBAH(0):
			sprintf(name, "TDWBAH0"); break;
		case E1000_TDBAL(1):
			sprintf(name, "TDBAL1"); break;
		case E1000_TDBAH(1):
			sprintf(name, "TDBAH1"); break;
		case E1000_TDLEN(1):
			sprintf(name, "TDLEN1"); break;
		case E1000_TDH(1):
			sprintf(name, "TDH1"); break;
		case E1000_DCA_TXCTRL(1):
			sprintf(name, "DCA_TXCTRL1"); break;
		case E1000_TDT(1):
			sprintf(name, "TDT1"); break;
		case E1000_TXDCTL(1):
			sprintf(name, "TXDCTL1"); break;
		case E1000_TDWBAL(1):
			sprintf(name, "TDWBAL1"); break;
		case E1000_TDWBAH(1):
			sprintf(name, "TDWBAH1"); break;
		case E1000_TDBAL(2):
			sprintf(name, "TDBAL2"); break;
		case E1000_TDBAH(2):
			sprintf(name, "TDBAH2"); break;
		case E1000_TDLEN(2):
			sprintf(name, "TDLEN2"); break;
		case E1000_TDH(2):
			sprintf(name, "TDH2"); break;
		case E1000_DCA_TXCTRL(2):
			sprintf(name, "DCA_TXCTRL2"); break;
		case E1000_TDT(2):
			sprintf(name, "TDT2"); break;
		case E1000_TXDCTL(2):
			sprintf(name, "TXDCTL2"); break;
		case E1000_TDWBAL(2):
			sprintf(name, "TDWBAL2"); break;
		case E1000_TDWBAH(2):
			sprintf(name, "TDWBAH2"); break;
		case E1000_TDBAL(3):
			sprintf(name, "TDBAL3"); break;
		case E1000_TDBAH(3):
			sprintf(name, "TDBAH3"); break;
		case E1000_TDLEN(3):
			sprintf(name, "TDLEN3"); break;
		case E1000_TDH(3):
			sprintf(name, "TDH3"); break;
		case E1000_DCA_TXCTRL(3):
			sprintf(name, "DCA_TXCTRL3"); break;
		case E1000_TDT(3):
			sprintf(name, "TDT3"); break;
		case E1000_TXDCTL(3):
			sprintf(name, "TXDCTL3"); break;
		case E1000_TDWBAL(3):
			sprintf(name, "TDWBAL3"); break;
		case E1000_TDWBAH(3):
			sprintf(name, "TDWBAH3"); break;
		case E1000_CRCERRS:
			if (!clr) continue;
			sprintf(name, "CRCERRS"); break;
		case E1000_ALGNERRC:
			if (!clr) continue;
			sprintf(name, "ALGNERRC"); break;
		case E1000_SYMERRS:
			if (!clr) continue;
			sprintf(name, "SYMERRS"); break;
		case E1000_RXERRC:
			if (!clr) continue;
			sprintf(name, "RXERRC"); break;
		case E1000_MPC:
			if (!clr) continue;
			sprintf(name, "MPC"); break;
		case E1000_SCC:
			if (!clr) continue;
			sprintf(name, "SCC"); break;
		case E1000_ECOL:
			if (!clr) continue;
			sprintf(name, "ECOL"); break;
		case E1000_MCC:
			if (!clr) continue;
			sprintf(name, "MCC"); break;
		case E1000_LATECOL:
			if (!clr) continue;
			sprintf(name, "LATECOL"); break;
		case E1000_COLC:
			if (!clr) continue;
			sprintf(name, "COLC"); break;
		case E1000_CBTMPC:
			if (!clr) continue;
			sprintf(name, "CBTMPC"); break;
		case E1000_DC:
			if (!clr) continue;
			sprintf(name, "DC"); break;
		case E1000_TNCRS:
			if (!clr) continue;
			sprintf(name, "TNCRS"); break;
		case E1000_SEC:
			if (!clr) continue;
			sprintf(name, "SEC"); break;
		case E1000_HTDPMC:
			if (!clr) continue;
			sprintf(name, "HTDPMC"); break;
		case E1000_RLEC:
			if (!clr) continue;
			sprintf(name, "RLEC"); break;
		case E1000_CBRDPC:
			if (!clr) continue;
			sprintf(name, "CBRDPC"); break;
		case E1000_XONRXC:
			if (!clr) continue;
			sprintf(name, "XONRXC"); break;
		case E1000_XONTXC:
			if (!clr) continue;
			sprintf(name, "XONTXC"); break;
		case E1000_XOFFRXC:
			if (!clr) continue;
			sprintf(name, "XOFFRXC"); break;
		case E1000_XOFFTXC:
			if (!clr) continue;
			sprintf(name, "XOFFTXC"); break;
		case E1000_FCRUC:
			if (!clr) continue;
			sprintf(name, "FCRUC"); break;
		case E1000_PRC64:
			if (!clr) continue;
			sprintf(name, "PRC64"); break;
		case E1000_PRC127:
			if (!clr) continue;
			sprintf(name, "PRC127"); break;
		case E1000_PRC255:
			if (!clr) continue;
			sprintf(name, "PRC255"); break;
		case E1000_PRC511:
			if (!clr) continue;
			sprintf(name, "PRC511"); break;
		case E1000_PRC1023:
			if (!clr) continue;
			sprintf(name, "PRC1023"); break;
		case E1000_PRC1522:
			if (!clr) continue;
			sprintf(name, "PRC1522"); break;
		case E1000_GPRC:
			if (!clr) continue;
			sprintf(name, "GPRC"); break;
		case E1000_BPRC:
			if (!clr) continue;
			sprintf(name, "BPRC"); break;
		case E1000_MPRC:
			if (!clr) continue;
			sprintf(name, "MPRC"); break;
		case E1000_GPTC:
			if (!clr) continue;
			sprintf(name, "GPTC"); break;
		case E1000_GORCL:
			if (!clr) continue;
			sprintf(name, "GORCL"); break;
		case E1000_GORCH:
			if (!clr) continue;
			sprintf(name, "GORCH"); break;
		case E1000_GOTCL:
			if (!clr) continue;
			sprintf(name, "GOTCL"); break;
		case E1000_GOTCH:
			if (!clr) continue;
			sprintf(name, "GOTCH"); break;
		case E1000_RNBC:
			if (!clr) continue;
			sprintf(name, "RNBC"); break;
		case E1000_RUC:
			if (!clr) continue;
			sprintf(name, "RUC"); break;
		case E1000_RFC:
			if (!clr) continue;
			sprintf(name, "RFC"); break;
		case E1000_ROC:
			if (!clr) continue;
			sprintf(name, "ROC"); break;
		case E1000_RJC:
			if (!clr) continue;
			sprintf(name, "RJC"); break;
		case E1000_MGTPRC:
			if (!clr) continue;
			sprintf(name, "MGTPRC"); break;
		case E1000_MGTPDC:
			if (!clr) continue;
			sprintf(name, "MGTPDC"); break;
		case E1000_MGTPTC:
			if (!clr) continue;
			sprintf(name, "MGTPTC"); break;
		case E1000_TORL:
			if (!clr) continue;
			sprintf(name, "TORL"); break;
		case E1000_TORH:
			if (!clr) continue;
			sprintf(name, "TORH"); break;
		case E1000_TOTL:
			if (!clr) continue;
			sprintf(name, "TOTL"); break;
		case E1000_TOTH:
			if (!clr) continue;
			sprintf(name, "TOTH"); break;
		case E1000_TPR:
			if (!clr) continue;
			sprintf(name, "TPR"); break;
		case E1000_TPT:
			if (!clr) continue;
			sprintf(name, "TPT"); break;
		case E1000_PTC64:
			if (!clr) continue;
			sprintf(name, "PTC64"); break;
		case E1000_PTC127:
			if (!clr) continue;
			sprintf(name, "PTC127"); break;
		case E1000_PTC255:
			if (!clr) continue;
			sprintf(name, "PTC255"); break;
		case E1000_PTC511:
			if (!clr) continue;
			sprintf(name, "PTC511"); break;
		case E1000_PTC1023:
			if (!clr) continue;
			sprintf(name, "PTC1023"); break;
		case E1000_PTC1522:
			if (!clr) continue;
			sprintf(name, "PTC1522"); break;
		case E1000_MPTC:
			if (!clr) continue;
			sprintf(name, "MPTC"); break;
		case E1000_BPTC:
			if (!clr) continue;
			sprintf(name, "BPTC"); break;
		case E1000_TSCTC:
			if (!clr) continue;
			sprintf(name, "TSCTC"); break;
		case E1000_CBRMPC:
			if (!clr) continue;
			sprintf(name, "CBRMPC"); break;
		case E1000_IAC:
			if (!clr) continue;
			sprintf(name, "IAC"); break;
		case E1000_RPTHC:
			if (!clr) continue;
			sprintf(name, "RPTHC"); break;
		case E1000_HGPTC:
			if (!clr) continue;
			sprintf(name, "HGPTC"); break;
		case E1000_ICRXDMTC:
			if (!clr) continue;
			sprintf(name, "ICRXDMTC"); break;
		case E1000_HTCBDPC:
			if (!clr) continue;
			sprintf(name, "HTCBDPC"); break;
		case E1000_HGORCL:
			if (!clr) continue;
			sprintf(name, "HGORCL"); break;
		case E1000_HGORCH:
			if (!clr) continue;
			sprintf(name, "HGORCH"); break;
		case E1000_HGOTCL:
			if (!clr) continue;
			sprintf(name, "HGOTCL"); break;
		case E1000_HGOTCH:
			if (!clr) continue;
			sprintf(name, "HGOTCH"); break;
		case E1000_LENERRS:
			if (!clr) continue;
			sprintf(name, "LENERRS"); break;
		case E1000_PCS_CFG0:
			if (!clr) continue;
			sprintf(name, "PCS_CFG0"); break;
		case E1000_PCS_LCTL:
			if (!clr) continue;
			sprintf(name, "PCS_LCTRL"); break;
		case E1000_PCS_LSTAT:
			if (!clr) continue;
			sprintf(name, "PCS_LSTAT"); break;
		case E1000_PCS_ANADV:
			if (!clr) continue;
			sprintf(name, "PCS_ANADV"); break;
		case E1000_PCS_LPAB:
			if (!clr) continue;
			sprintf(name, "PCS_LPAB"); break;
		case E1000_PCS_NPTX:
			if (!clr) continue;
			sprintf(name, "PCS_NPTX"); break;
		case E1000_PCS_LPABNP:
			if (!clr) continue;
			sprintf(name, "PCS_LPABNP"); break;
		case E1000_SCVPC:
			if (!clr) continue;
			sprintf(name, "SCVPC"); break;
		case E1000_RXCSUM:
			sprintf(name, "RXCSUM"); break;
		case E1000_RFCTL:
			sprintf(name, "RFCTL"); break;
		case E1000_MTA:
			sprintf(name, "MTA");
			printit32_array(i, name, mem, 128);
			continue;
		case E1000_RA:
			for (j=i, n=0; n<16; n++) {
				sprintf(name, "RAL[%d]", n);
				printit32(j, name, mem);
				j += sizeof(u32);
				sprintf(name, "RAH[%d]", n);
				printit32(j, name, mem);
				j += sizeof(u32);
			}
			continue;
		case E1000_PSRTYPE(0):
			for (j=i, n=0; n<4; n++) {
				sprintf(name, "PSRTYPE[%d]", n);
				printit32(j, name, mem);
				j += sizeof(u32);
			}
		case E1000_VFTA:
			sprintf(name, "VFTA");
			printit32_array(i, name, mem, 128);
			continue;
		case E1000_WUC:
			sprintf(name, "WUC"); break;
		case E1000_WUFC:
			sprintf(name, "WUFC"); break;
		case E1000_WUS:
			sprintf(name, "WUS"); break;
		case E1000_MRQC:
			sprintf(name, "MRQC"); break;
		case E1000_VMD_CTL:
			sprintf(name, "VMD_CTL"); break;
		case E1000_MANC:
			sprintf(name, "MANC"); break;
		case E1000_IPAV:
			sprintf(name, "IPAV"); break;
		case E1000_MANC2H:
			sprintf(name, "MANC2H"); break;
		case E1000_WUPL:
			if (!clr) continue;
			sprintf(name, "WUPL"); break;
		case E1000_IMIR(0):
			sprintf(name, "IMIR");
			printit32_array(i, name, mem, 8);
			continue;
		case E1000_IMIREXT(0):
			sprintf(name, "IMIREX");
			printit32_array(i, name, mem, 8);
			continue;
		case E1000_IMIRVP:
			sprintf(name, "IMIRVP"); break;
		case E1000_GCR:
			sprintf(name, "GCR"); break;
		case E1000_GSCL_1:
			sprintf(name, "GSCL_1"); break;
		case E1000_GSCL_2:
			sprintf(name, "GSCL_2"); break;
		case E1000_GSCL_3:
			sprintf(name, "GSCL_3"); break;
		case E1000_GSCL_4:
			sprintf(name, "GSCL_4"); break;
		case E1000_FACTPS:
			sprintf(name, "FACTPS"); break;
		case E1000_FWSM:
			sprintf(name, "FWSM"); break;
		case E1000_SW_FW_SYNC:
			sprintf(name, "SW_FW_SYNC"); break;
		case E1000_DCA_ID:
			sprintf(name, "DCA_ID"); break;
		case E1000_DCA_CTRL:
			sprintf(name, "DCA_CTRL"); break;
		case E1000_RETA(0):
			for (j=i, n=0; n<128; n++) {
				sprintf(name, "RETA[%d]", n);
				printit8(j, name, mem);
				j += sizeof(u8);
			}
			continue;
		case E1000_RSSRK(0):
			for (j=i, n=0; n<40; n++) {
				sprintf(name, "RSSRK[%d]", n);
				printit8(j, name, mem);
				j += sizeof(u8);
			}
			continue;
		case E1000_FFLT:
			sprintf(name, "FFLT");
			printit32_array(i, name, mem, 4);
			continue;
		case E1000_HICR:
			sprintf(name, "HICR"); break;
		case E1000_VFQA0:
			sprintf(name, "VFQA0"); break;
			sprintf(name, "VFQA0");
			printit32_array(i, name, mem, 128);
			continue;
		case E1000_VFQA1:
			sprintf(name, "VFQA1"); break;
			sprintf(name, "VFQA1");
			printit32_array(i, name, mem, 128);
			continue;
		default:
			continue;
		}
		printit32(i, name, mem);
	}
}

// vi:ts=4:sw=4:ht=4:
